# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import cairo, math
from decimal import Decimal
from glogic import const
from glogic.Utils import *
from gi.repository import Gtk, Pango, PangoCairo
from gettext import gettext as _

class ChartArea(Gtk.Box):
	def __init__(self, circuit):
		Gtk.Box.__init__(self)
		
		self.circuit = circuit
		self.scale = 5000000 # pix/sec
		self.start_time = 0.0
		self.end_time = 0.0002
		self.chart_unit = 1 # 0: ns, 1: µs, 2: ms
		
		self.gra_pix = 120
		
		self.name_width = 150
		
		self.chart_scroll = Gtk.ScrolledWindow()
		self.chart_scroll.set_hexpand(True)
		self.chart_scroll.set_vexpand(True)
		self.chartarea = Gtk.DrawingArea()
		self.chart_scroll.add_with_viewport(self.chartarea)
		
		self.name_scroll = Gtk.ScrolledWindow(vadjustment=self.chart_scroll.get_vadjustment())
		self.name_scroll.set_size_request(150, -1)
		self.namearea = Gtk.DrawingArea()
		self.name_scroll.add_with_viewport(self.namearea)
		
		self.pack_start(self.name_scroll, False, False, 0)
		self.pack_start(self.chart_scroll, True, True, 0)
		
		self.cursor_x = 0
		self.cursor_y = 0
		self.drawing_font = Pango.FontDescription("VL Gothic 10")
		self.name_pixbuf = None
		self.chart_pixbuf = None

		self.namearea.connect("draw", self.on_namearea_draw)
		self.chartarea.connect("draw", self.on_chartarea_draw)

	def on_namearea_draw(self, widget, cr):
		if self.name_pixbuf is not None:
			cr.set_source_surface(self.name_pixbuf, 0, 0);
			cr.paint();

	def on_chartarea_draw(self, widget, cr):
		if self.chart_pixbuf is not None:
			cr.set_source_surface(self.chart_pixbuf, 0, 0);
			cr.paint();
	
	def draw_names(self, cr):
		name_layout = PangoCairo.create_layout(cr)
		name_layout.set_font_description(self.drawing_font)
		# draw borders
		cr.set_source_rgb(1.0, 1.0, 1.0)
		cr.rectangle(1, 1, self.name_width-1, 38)
		cr.fill()
		cr.set_source_rgb(0.875, 0.875, 0.875)
		cr.rectangle(1, 40, self.name_width-1, self.img_height-41)
		cr.fill()
		cr.set_source_rgb(0.5, 0.5, 0.5)
		cairo_paths(cr, (0, 0.5), (self.name_width, 0.5))
		cairo_paths(cr, (0.5, 1), (0.5, self.img_height))
		cairo_paths(cr, (0, 39.5), (self.name_width, 39.5))
		i = 0
		for c in self.circuit.components:
			if c[0] == const.component_probe:
				cairo_paths(cr, (1, i * 40 + 79.5), (self.name_width, i * 40 + 79.5))
				i += 1
		cr.stroke()
		
		# draw probe names
		cr.set_source_rgb(0.0, 0.0, 0.0)
		i = 0
		for c in self.circuit.components:
			if c[0] == const.component_probe:
				cairo_draw_text(cr, name_layout, c[1].values[0], 10, i * 40 + 50)
				i += 1
		cr.fill()
		
		if self.chart_unit == 0:
			cairo_draw_text(cr, name_layout, "%s [ns]" % _("Time"), 130, 35, 1.0, 1.0)
		elif self.chart_unit == 1:
			cairo_draw_text(cr, name_layout, "%s [µs]" % _("Time"), 130, 35, 1.0, 1.0)
		else:
			cairo_draw_text(cr, name_layout, "%s [ms]" % _("Time"), 130, 35, 1.0, 1.0)
			
		cr.fill()
	
	def draw_charts(self, cr):
		chart_layout = PangoCairo.create_layout(cr)
		chart_layout.set_font_description(self.drawing_font)
		
		# draw borders
		cr.set_source_rgb(1.0, 1.0, 1.0)
		cr.rectangle(1, 1, self.chart_width-2, self.img_height-2)
		cr.fill()
		cr.set_source_rgb(0.5, 0.5, 0.5)
		cr.rectangle(0.5, 0.5, self.chart_width-1, self.img_height-1)
		cairo_paths(cr, (1, 39.5), (self.chart_width-1, 39.5))
		i = 0
		for c in self.circuit.components:
			if c[0] == const.component_probe:
				cairo_paths(cr, (1, i * 40 + 79.5), (self.chart_width-1, i * 40 + 79.5))
				i += 1
		cr.stroke()
		
		# draw graduaton
		gra_p = Decimal(self.gra_pix) / Decimal(self.scale)
		order = 10 ** Decimal(int(math.log10(gra_p)))
		grasec_top = gra_p / order
		if 0.15 <= grasec_top < 0.35:
			gra_p = order * Decimal("0.2")
		elif 0.35 <= grasec_top <= 0.75:
			gra_p = order * Decimal("0.5")
		else:
			gra_p = order * Decimal("0.1")
		cr.set_source_rgb(0.0, 0.0, 0.0)
		
		if self.chart_unit == 0:
			u_start_time = self.start_time * 1000000000
			u_end_time = self.end_time * 1000000000
			u_gra_p = gra_p * 1000000000
		elif self.chart_unit == 1:
			u_start_time = self.start_time * 1000000
			u_end_time = self.end_time * 1000000
			u_gra_p = gra_p * 1000000
		else:
			u_start_time = self.start_time * 1000
			u_end_time = self.end_time * 1000
			u_gra_p = gra_p * 1000
		
		cairo_draw_text(cr, chart_layout, str(Decimal(u_start_time).quantize(Decimal(".00"))), 5, 35, 0.0, 1.0)
		cairo_draw_text(cr, chart_layout, str(Decimal(u_end_time).quantize(Decimal(".00"))), self.chart_width - 5, 35, 1.0, 1.0)
		cr.fill()
		
		u_gra_sec = u_gra_p
		gra_sec = gra_p
		while True:
			if u_gra_sec + u_gra_p / 4 >= u_end_time:
				break
			cr.set_source_rgb(0.5, 0.5, 0.5)
			cairo_paths(cr, (int(float(gra_sec) * self.scale) + 0.5, 39), (int(float(gra_sec) * self.scale) + 0.5, self.img_height-1))
			cr.stroke()
			cr.set_source_rgb(0.0, 0.0, 0.0)
			cairo_draw_text(cr, chart_layout, str(Decimal(u_gra_sec).quantize(Decimal(".00"))), float(gra_sec) * self.scale, 35, 0.5, 1.0)
			u_gra_sec += u_gra_p
			gra_sec += gra_p
			cr.fill()
		
		# plot graphs
		prev_time = 0
		history = self.circuit.probe_levels_history
		for i, probe_lebels in enumerate(history[:-1]):
			if history[i + 1][0] <= self.start_time or self.end_time < probe_lebels[0]:
				continue
			for j, data in enumerate(probe_lebels[1:]):
				if data == True:
					cairo_paths(cr, (round((probe_lebels[0] - self.start_time) * self.scale), j * 40 + 55.5), (round((history[i + 1][0] - self.start_time) * self.scale) + 1, j * 40 + 55.5))
				else:
					cairo_paths(cr, (round((probe_lebels[0] - self.start_time) * self.scale), j * 40 + 65.5), (round((history[i + 1][0] - self.start_time) * self.scale) + 1, j * 40 + 65.5))
				if data != history[i + 1][j + 1]:
					cairo_paths(cr, (round((history[i + 1][0] - self.start_time) * self.scale) + 0.5, j * 40 + 56), (round((history[i + 1][0] - self.start_time) * self.scale) + 0.5, j * 40 + 65))
		
		if history[-1][0] < self.end_time:
			for j, data in enumerate(history[-1][1:]):
				if data == True:
					cairo_paths(cr, (round((history[-1][0] - self.start_time) * self.scale), j * 40 + 55.5), (self.chart_width, j * 40 + 55.5))
				else:
					cairo_paths(cr, (round((history[-1][0] - self.start_time) * self.scale), j * 40 + 65.5), (self.chart_width, j * 40 + 65.5))
				
		cr.stroke()
		
	def createChart(self):
		i = 0
		for c in self.circuit.components:
			if c[0] == const.component_probe:
				i += 1
		self.img_height = 40 * i + 40
		
		self.chart_width = int((self.end_time - self.start_time) * self.scale)
		if self.chart_width == 0:
			self.chart_width = 1
			
		self.name_pixbuf = cairo.ImageSurface(cairo.FORMAT_RGB24, self.name_width, self.img_height)
		self.namearea.set_size_request(self.name_width, self.img_height)
		ncr = cairo.Context(self.name_pixbuf)
		ncr.set_line_width(1.0)
		self.draw_names(ncr)
		
		self.chart_pixbuf = cairo.ImageSurface(cairo.FORMAT_RGB24, self.chart_width, self.img_height)
		self.chartarea.set_size_request(self.chart_width, self.img_height)
		ccr = cairo.Context(self.chart_pixbuf)
		ccr.set_line_width(1.0)
		self.draw_charts(ccr)
		self.queue_draw()
