# -*- coding: utf-8; indent-tabs-mode: t; tab-width: 4 -*-

import math
from gettext import gettext as _
from glogic import const
from glogic.Utils import *

class BaseComponent():
	def __init__(self, *args, **kwds):
		self.pos_x = 0
		self.pos_y = 0
		self.mouse_button = False
		self.tp_lh = 0
		self.tp_hl = 0
		self.properties = []
		self.values = []
		self.prop_names = []

	def drawComponent(self, cr, layout, running_mode):
		return

	def isMouseOvered(self, x, y):
		return False

	def isInRectangle(self, x1, y1, x2, y2):
		return False
	
	def mouse_down(self, x, y):
		if self.isMouseOvered(x, y):
			self.mouse_button = True
	
	def mouse_up(self, x, y):
		if self.mouse_button:
			self.mouse_button = False
			if self.isMouseOvered(x, y):
				return self.click(x, y)
		return False
	
	def click(self, x, y):
		return False
	
	def propertyChanged(self, prop):
		return False

	def calculate(self, input_datas):
		return []
	
	def set_output(self, pin, level):
		return

class DummyComponent(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.input_pins = []
		self.output_pins = []
		self.input_level = []
		self.output_level = []

	def drawComponent(self, x, y, dc, adc, running_mode):
		return

	def isMouseOvered(self, x, y):
		return False

	def isInRectangle(self, x1, y1, x2, y2):
		return False

class OSC(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -40, 70, 0]
		self.input_pins = []
		self.output_pins = [(70, -20)]
		self.input_level = []
		self.output_level = [False]
		self.prop_names = ["period", "shift"]
		self.properties.append((_("Period:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		self.properties.append((_("Shift:"), (const.property_float, 0, 1000, 3, 100), "µs"))
		self.values.append(0)
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.output_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (50, -20), (70, -20))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (50, -20), (70, -20))
		cairo_paths(cr, (15, -10), (20, -10), (20, -30), (25, -30), (25, -10), (30, -10), (30, -30), (35, -30), (35, -10), (40, -10), (40, -30), (45, -30))
		cr.stroke()
		cr.rectangle(10, -40, 40, 40)
		cr.stroke()
		
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 67 and self.pos_y - 37 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 67 <= x2 and y1 <= self.pos_y - 37 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		return [not self.output_level[0]]

class Probe(BaseComponent):
	def __init__(self, *args, **kwds):
		BaseComponent.__init__(self, *args, **kwds)
		self.comp_rect = [10, -20, 55, 0]
		self.input_pins = [(10, -10)]
		self.output_pins = []
		self.input_level = [False]
		self.output_level = []
		self.prop_names = ["name"]
		self.properties.append((_("Name:"), (const.property_string, 20), ""))
		self.values.append("Probe")
		self.width = 12
		
	def drawComponent(self, cr, layout, running_mode):
		if running_mode:
			tmpsrc = cr.get_source()
			if self.input_level[0]:
				cr.set_source_rgb(1.0, 0.0, 0.0)
			else:
				cr.set_source_rgb(0.0, 0.0, 1.0)
			cairo_paths(cr, (10, -10), (30, -10))
			cr.stroke()
			cr.set_source(tmpsrc)
		else:
			cairo_paths(cr, (10, -10), (30, -10))
			cr.stroke()
		cr.arc(40, -10, 10, 0, 2 * math.pi)
		cr.stroke()
		cairo_draw_text(cr, layout, "V", 40, -10, 0.5, 0.5)
		cairo_draw_text(cr, layout, self.values[0], 55, -10, 0.0, 0.5)
		(w, h) = layout.get_size()
		self.width = w / Pango.SCALE
		if self.width < 12:
			self.width = 12
		self.comp_rect = [10, -20, 55 + self.width, 0]
	
	def isMouseOvered(self, x, y):
		if self.pos_x + 13 <= x <= self.pos_x + 52 + self.width and self.pos_y - 17 <= y <= self.pos_y - 3:
			return True
		return False
	
	def isInRectangle(self, x1, y1, x2, y2):
		if x1 <= self.pos_x + 13 and self.pos_x + 55 + self.width <= x2 and y1 <= self.pos_y - 17 and self.pos_y - 3 <= y2:
			return True
		return False
	
	def calculate(self, input_datas):
		self.input_level[0] = input_datas[0]
		return []

